package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesAlert;
    import com.despark.tuning3d.view.components.challenges.modals.*;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;
    import com.utils.Tracker;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

public class ChallengeNotificationsPopupMediator extends PopUpMediator {
    private var _proxy:ChallengesProxy;
    private var _notificationPopUp:ChallengeNotificationsPopup;
    public function ChallengeNotificationsPopupMediator(popUp:PopUp) {
        super(popUp);
        _notificationPopUp = popUp as ChallengeNotificationsPopup;
    }

    override public function onRegister():void {
        super.onRegister();
        _proxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;

        _notificationPopUp.addEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);

        _notificationPopUp.addEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineNotificationHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_CHALLENGE_TUNING, viewChallengeTuningHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_REQUEST_TUNING, tuningSelectedHandler);

        _notificationPopUp.addEventListener(StackedScreen.CHALLENGES_SELECTED, displayChallengesHandler);
        _notificationPopUp.addEventListener(StackedScreen.TUNING_SELECTED, userTuningSelectedHandler);


        _proxy.markAllUnreadNotifications(function ():void {
            sendNotification(Notifications.CHALLENGE_NOTIFICATIONS_VIEWED);
        });
        //
        Starling.juggler.delayCall(function():void{
            BannerStackedScreen.hideAdBanner(false);
        }, 0.2);
        Tracker.trackEvent("VS Notifications Popup", "open");
    }

    override public function onRemove():void {
        super.onRemove();
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);

        _notificationPopUp.removeEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineNotificationHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_CHALLENGE_TUNING, viewChallengeTuningHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_REQUEST_TUNING, tuningSelectedHandler);

        _notificationPopUp.removeEventListener(StackedScreen.CHALLENGES_SELECTED, displayChallengesHandler);
        _notificationPopUp.removeEventListener(StackedScreen.TUNING_SELECTED, userTuningSelectedHandler);
    }

    private function tuningSelectedHandler(event:Event):void {
        _notificationPopUp.close();
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.closeHandler =  closeHandler;
        sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_TUNING_POPUP, data);
    }

    private function viewUserTuningsHandler(event:Event):void {
        // _notificationPopUp.close();
        Constants.selectedUser = event.data.user;
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        // sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler});
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    private function viewChallengeTuningHandler(event:Event):void {
        _notificationPopUp.close();
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler, data:event.data});
    }



    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.CHALLENGE_ACCEPTED_NOTIFICATION,
            Notifications.CHALLENGE_DECLINED_NOTIFICATION,
            Notifications.CHALLENGE_CREATE_ERROR
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.CHALLENGE_ACCEPTED_NOTIFICATION:
                handlerAcceptChallenge(note.getBody());
                break;
            case Notifications.CHALLENGE_DECLINED_NOTIFICATION:
                handlerDeclineChallenge(note.getBody());
                break;

            case Notifications.CHALLENGE_CREATE_ERROR:
                /*if(!note.getBody().hasOwnProperty("shown") || !note.getBody()["shown"]){
                    handleStartChallengeDenied(note.getBody());
                }
                note.getBody().shown = true;*/
                break;
        }
        super.handleNotification(note);
    }

    private function handlerDeclineChallenge(body:Object):void {
        _notificationPopUp.removeItem(body.item);
    }

    private function handlerAcceptChallenge(body:Object):void {
        _notificationPopUp.removeItem(body.item);
    }

    private function getRequestsHandler(event:Event):void {
        _proxy.getRequests(event.data.skip, event.data.limit, _notificationPopUp.loadInvitations);
    }

    private function getNotificationsHandler(event:Event):void {
        _proxy.getNotifications(event.data.skip, event.data.limit, _notificationPopUp.loadNotifications);
    }

    private function handleChallengeRequest(event:Event):void {
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function declineNotificationHandler(event:Event):void {
        sendNotification(Notifications.CHALLENGE_DECLINED_NOTIFICATION, {item:event.data.notification});
    }

    private function displayChallengesHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
        popUpCloseButtonSelectedHandler(null);
    }

    private function userTuningSelectedHandler(event:Event):void {
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.eventDispatcher = _notificationPopUp;
        /*data.closeHandler = function():void{
            sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP);
        };*/

        var selectedUser:User = data.user;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var currentUser:User = usersProxy.currentUser;
        if((currentUser && selectedUser && selectedUser.id != currentUser.id) || !currentUser){
            Constants.selectedUser = selectedUser;
        }else{
            Constants.selectedUser = null;
        }
        sendNotification(Notifications.DISPLAY_GALLERY_TUNING_POPUP, data);
    }

    private function handleStartChallengeDenied(data:Object):void {
        if(data.statusCode == ChallengeErrorCode.MINE_IN_CHALLENGE_STATUS_CODE){
            ChallengesAlert.show(ChallengesAlert.DEFAULT);
        }else if(data.statusCode == ChallengeErrorCode.MAX_COUNT_STATUS_CODE){
            ChallengesAlert.show(ChallengesAlert.LIMIT);
        }else if(data.statusCode == ChallengeErrorCode.NOT_ENOUGH_COIN_CODE){
            ChallengesAlert.show(ChallengesAlert.REWARDS, _notificationPopUp);
        }
    }

}
}
