package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.ChallengesListScreen;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengesAlert;
    import com.despark.tuning3d.view.components.challenges.modals.ChallengeErrorCode;
    import com.despark.tuning3d.view.components.challenges.modals.StartChallengePopup;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;

    import feathers.core.PopUpManager;

    import org.puremvc.as3.interfaces.INotification;
import starling.events.Event;

public class StartChallengePopupMediator extends PopUpMediator{
    private var _currentUser:User;
    public function StartChallengePopupMediator(popUp:PopUp) {
        super(popUp);
    }

    override public function onRegister():void {
        super.onRegister();
        startChallengePopup.addEventListener(ChallengeEvent.LOAD_USER_CHALLENGES, loadCurrentUserTunings);
        startChallengePopup.addEventListener(ChallengeEvent.LOAD_USER_FRIENDS, loadCurrentUserFriends);
        startChallengePopup.addEventListener(ChallengeEvent.LOAD_USER_ALLOWED_TUNINGS, loadTuningsHandler);
        startChallengePopup.addEventListener(ChallengeEvent.SEARCH_USERS, searchUsersHandler);
        startChallengePopup.addEventListener(ChallengeEvent.SEARCH_TUNINGS, searchTuningsHandler);
        startChallengePopup.addEventListener(ChallengeEvent.INVITE_FOR_CHALLENGE, handleInviteForChallenge);
        startChallengePopup.addEventListener(ChallengeEvent.RANDOM_CHALLENGE, handleRandomChallenge);

        startChallengePopup.addEventListener(ChallengeEvent.ACCEPT_CHALLENGE, acceptChallengeHandler);
        startChallengePopup.addEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineChallengeHandler);

        startChallengePopup.addEventListener(StackedScreen.CHALLENGES_SELECTED, displayChallengesHandler);
    }

    override public function onRemove():void {
        super.onRemove();
        startChallengePopup.removeEventListener(ChallengeEvent.LOAD_USER_CHALLENGES, loadCurrentUserTunings);
        startChallengePopup.removeEventListener(ChallengeEvent.LOAD_USER_FRIENDS, loadCurrentUserFriends);
        startChallengePopup.removeEventListener(ChallengeEvent.LOAD_USER_ALLOWED_TUNINGS, loadTuningsHandler);
        startChallengePopup.removeEventListener(ChallengeEvent.SEARCH_USERS, searchUsersHandler);
        startChallengePopup.removeEventListener(ChallengeEvent.SEARCH_TUNINGS, searchTuningsHandler);
        startChallengePopup.removeEventListener(ChallengeEvent.INVITE_FOR_CHALLENGE, handleInviteForChallenge);
        startChallengePopup.removeEventListener(ChallengeEvent.RANDOM_CHALLENGE, handleRandomChallenge);

        startChallengePopup.removeEventListener(ChallengeEvent.ACCEPT_CHALLENGE, acceptChallengeHandler);
        startChallengePopup.removeEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineChallengeHandler);

        startChallengePopup.removeEventListener(StackedScreen.CHALLENGES_SELECTED, displayChallengesHandler);
    }

    protected function get startChallengePopup():StartChallengePopup{
        return StartChallengePopup(popUp);
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.START_CHALLENGE,
            Notifications.CHALLENGES_LOADED,
            Notifications.CHALLENGE_TUNINGS_LOADED,
            Notifications.CHALLENGE_FRIENDS_LOADED,
            Notifications.CHALLENGE_CREATE_SUCCESS,
            Notifications.CHALLENGE_CREATE_ERROR
        ]);
    }

    public function loadCurrentUserTunings(event:Event):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _currentUser = usersProxy.currentUser;
        if(_currentUser) {
            var challengesProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengesProxy.loadUserChallenges(_currentUser.id, event.data.selectTuning);
        }else{
            // TODO
        }
    }

    private function displayChallengesHandler(event:Event):void {
        PopUpManager.removeAllPopUps(true);
        sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
    }

    public function loadCurrentUserFriends(event:Event):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _currentUser = usersProxy.currentUser;
        if(_currentUser) {
            var challengesProxy:ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengesProxy.loadFriends(_currentUser, event.data.skip, ChallengesProxy.USER_REQUEST_COUNT);
        }else{
            // TODO
        }
    }

    public function searchUsersHandler(event:Event):void{
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        challengesProxy.searchUsers(event.data.searchTerm, event.data.skip, event.data.completeHandler, ChallengesProxy.USER_REQUEST_COUNT);
    }

    public function searchTuningsHandler(event:Event):void{
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        challengesProxy.searchTunings(event.data.searchTerm, event.data.skip, ChallengesProxy.USER_REQUEST_COUNT, event.data.completeHandler);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.START_CHALLENGE:
                handleStartChallengeReady(note.getBody());
                break;
            case Notifications.CHALLENGES_LOADED:
                handleChallengesLoaded(note.getBody());
                break;
            case Notifications.CHALLENGE_TUNINGS_LOADED:
                handleTuningsLoaded(note.getBody());
                break;
            case Notifications.CHALLENGE_FRIENDS_LOADED:
                handleFriendsLoaded(note.getBody());
                break;

            case Notifications.CHALLENGE_CREATE_SUCCESS:
                handleChallengeStartSuccess();
                break;

            case Notifications.CHALLENGE_CREATE_ERROR:
                if(!note.getBody().hasOwnProperty("shown") || !note.getBody()["shown"]){
                    handleStartChallengeDenied(note.getBody());
                }
                note.getBody().shown = true;
                break;

        }
        super.handleNotification(note);
    }

    private function handleInviteForChallenge(event:Event):void {
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var data:Object = event.data;
        challengesProxy.inviteForChallenge(data);
    }

    private function handleRandomChallenge(event:Event):void {
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var data:Object = event.data;
        challengesProxy.randomChallenge(data.userId, data.carId, data.carType, data.challengeType, data.tuningId);
    }

    private function acceptChallengeHandler(event:Event):void {
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var data:Object = event.data;
        challengesProxy.acceptChallenge(data.challengeId, data.tuningId, true);

        sendNotification(Notifications.CHALLENGE_ACCEPTED_NOTIFICATION, {item:data.notification});
    }

    private function declineChallengeHandler(event:Event):void {
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var data:Object = event.data;
        challengesProxy.acceptChallenge(data.challengeId, data.tuningId, false);

        sendNotification(Notifications.CHALLENGE_DECLINED_NOTIFICATION, {item:data.notification});
    }

    private function handleChallengesLoaded(body:Object):void {
        var result:Array = [];
        var challenges:Array = body.challenges as Array;
        for(var i:int = 0; i < challenges.length; i++){
            if(challenges[i].owner.user == _currentUser.id){
                result.push(challenges[i].owner.tuning);
            }else if(challenges[i].receiver.user == _currentUser.id){
                result.push(challenges[i].receiver.tuning);
            }
        }
        startChallengePopup.appendChallenges(result);
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        challengesProxy.loadTunings(_currentUser, body.selectTuning, startChallengePopup.tuningSkip, startChallengePopup.tuningLimit);
    }

    private function loadTuningsHandler(event:Event):void{
        var challengesProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        challengesProxy.loadTunings(_currentUser, false, startChallengePopup.tuningSkip, startChallengePopup.tuningLimit);
    }

    private function handleTuningsLoaded(body:Object):void{
        startChallengePopup.setUserTunings(body.tunings, body.selectTuning, false);
    }

    private function handleFriendsLoaded(body:Object):void{
        startChallengePopup.setUserFriends(body.friends);
    }

    private function handleChallengeStartSuccess():void{
        startChallengePopup.close();
    }

    private function handleStartChallengeReady(body:Object):void {
        trace(body);
    }

    private function handleStartChallengeDenied(data:Object):void {
        if(data.statusCode == ChallengeErrorCode.MINE_IN_CHALLENGE_STATUS_CODE){
            ChallengesAlert.show(ChallengesAlert.MINE_IN_CHALLENGE, null, data.message, startChallengePopup.enablePopup);
        }else if(data.statusCode == ChallengeErrorCode.OPPONENT_IN_CHALLENGE_STATUS_CODE){
            ChallengesAlert.show(ChallengesAlert.OPPONENT_IN_CHALLENGE, null, data.message, startChallengePopup.enablePopup);
        }else if(data.statusCode == ChallengeErrorCode.MAX_COUNT_STATUS_CODE){
            ChallengesAlert.show(ChallengesAlert.LIMIT, null, null, startChallengePopup.enablePopup);
        }else if(data.statusCode == ChallengeErrorCode.NOT_ENOUGH_COIN_CODE){
            ChallengesAlert.show(ChallengesAlert.REWARDS, startChallengePopup, null, startChallengePopup.enablePopup);
        }
    }
}
}
