package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.challenges.feed.ChallengeAcceptedNoteData;
import com.despark.tuning3d.model.challenges.feed.ChallengeLostNoteData;
import com.despark.tuning3d.model.challenges.feed.ChallengeNotificationData;
import com.despark.tuning3d.model.challenges.feed.ChallengeRejectedNoteData;
import com.despark.tuning3d.model.challenges.feed.ChallengeRequestNoteData;
import com.despark.tuning3d.model.challenges.feed.ChallengeWonNoteData;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.utils.Console;

    import feathers.controls.LayoutGroup;

import flash.display.BitmapData;
import flash.geom.Rectangle;

import starling.core.Starling;
import starling.display.DisplayObject;
import starling.display.DisplayObjectContainer;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.DropShadowFilter;
import starling.textures.Texture;

public class ChallengeNotificationItemRenderer extends ChallengeListItemRenderer {
    private var _eventDispatcher:EventDispatcher;
    private var _challengeNoteData:ChallengeNotificationData;
    private var _icon:Image;
    private var _iconTailRight:Quad;
    private var _background:Quad;
    private var _bgBottomBorder:Quad;
    private var _contentBorder:Image;
    private var _dateLabel:DateLabel;
    private static var borderTexture:Texture;
    public function ChallengeNotificationItemRenderer() {
        super();
        createBorderTexture();
    }

    private function touchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        if(touch){
            Console.log();
            if(_challengeNoteData.disposed){
                _challengeNoteData.createView();
                invalidate();
            }
        }
    }

    override protected function draw():void {
        super.draw();
        updateView();
    }

    public function updateView():void{
        if(_background) {
            _background.width = actualWidth;
            _background.height = actualHeight - 4;
            _background.y = (actualHeight - _background.height) / 2;
        }

        if(_bgBottomBorder) {
            _bgBottomBorder.width = actualWidth;
            _bgBottomBorder.height = 0.8;
            _bgBottomBorder.y = _background.y + _background.height;
        }

        var leftMargin:Number = Device.isIphoneXVariant()?90:60;
        var rightMargin:Number = Device.isIphoneXVariant()?45:12;
        var iconRightMargin:Number = 16;
        if(_icon) {
            _icon.x = leftMargin - (_icon.width + iconRightMargin);
            _icon.y = (actualHeight - _icon.height) / 2;

            if(_contentBorder){
                _contentBorder.width = actualWidth - (leftMargin + rightMargin); //(_icon.width+10);
                _contentBorder.height = actualHeight - 20;
                _contentBorder.x = leftMargin;
                _contentBorder.y = (actualHeight - _contentBorder.height)/2;
            }

            if(_iconTailRight == null){
                _iconTailRight = new Quad(1,1, 0xe8e8e8);
                _iconTailRight.width = iconRightMargin;
                _iconTailRight.height = 1;
                addChild(_iconTailRight);
                _iconTailRight.x = leftMargin - _iconTailRight.width;
                _iconTailRight.y = _icon.y + (_icon.height - _iconTailRight.height) / 2;
            }else if(!contains(_iconTailRight)){
                addChild(_iconTailRight);
            }
            setChildIndex(_iconTailRight, numChildren - 1);

            if(_dateLabel){
                _dateLabel.time = _challengeNoteData.time;
                _dateLabel.x = _contentBorder.x + 5;
                _dateLabel.y = _contentBorder.y+_contentBorder.height - (_dateLabel.height + 5);
            }

            if(_challengeNoteData.messageLabel){
                _challengeNoteData.messageLabel.width = _contentBorder.width - leftMargin; // 20
                validateLayoutGroup(_challengeNoteData.messageLabel);

                _challengeNoteData.messageLabel.x = _dateLabel.x;
                _challengeNoteData.messageLabel.y = (actualHeight - (_challengeNoteData.messageLabel.height + _dateLabel.height)) / 2;//Math.min(_contentBorder.y + 20, _dateLabel.y-_challengeNoteData.messageLabel.height);

                _dateLabel.y = _challengeNoteData.messageLabel.y + _challengeNoteData.messageLabel.height + 2;
            }

            if(data is ChallengeRequestNoteData){
                if (_challengeNoteData.avatar) {
                    _challengeNoteData.avatar.x = _contentBorder.x + 10;
                    _challengeNoteData.avatar.y = (actualHeight - _challengeNoteData.avatar.height) / 2;
                }
                if (_challengeNoteData.buttonGroup) {
                    _challengeNoteData.buttonGroup.validate();
                    _challengeNoteData.buttonGroup.height = actualHeight;
                    _challengeNoteData.buttonGroup.x = _contentBorder.x + _contentBorder.width - (_challengeNoteData.buttonGroup.width + (_challengeNoteData.tuningView?_challengeNoteData.tuningView.width:10));
                    _challengeNoteData.buttonGroup.validate();
                }

                if (_challengeNoteData.tuningView) {
                    _challengeNoteData.tuningView.height = _contentBorder.height;
                    validateLayoutGroup(_challengeNoteData.tuningView);

                    _challengeNoteData.tuningView.x = _contentBorder.x + _contentBorder.width - _challengeNoteData.tuningView.width;
                    _challengeNoteData.tuningView.y = _contentBorder.y + (_contentBorder.height - _challengeNoteData.tuningView.height) / 2;
                }
                if(_challengeNoteData.messageLabel) {
                    var mw:Number = _contentBorder.width - (30+_challengeNoteData.buttonGroup.width + (_challengeNoteData.tuningView?_challengeNoteData.tuningView.width:0) + _challengeNoteData.avatar.width);
                    _challengeNoteData.messageLabel.width = mw;
                    validateLayoutGroup(_challengeNoteData.messageLabel);
                    _challengeNoteData.messageLabel.x = _challengeNoteData.avatar.x + _challengeNoteData.avatar.width + 5;
                    _challengeNoteData.messageLabel.y = (actualHeight - (_challengeNoteData.messageLabel.height + _dateLabel.height)) / 2; //Math.min(_contentBorder.y + 20, _dateLabel.y-_challengeNoteData.messageLabel.height);
                }

                if(_dateLabel && _challengeNoteData.avatar) {
                    _dateLabel.x = _challengeNoteData.avatar.x + _challengeNoteData.avatar.width + 5;
                    _dateLabel.y = _challengeNoteData.messageLabel.y + _challengeNoteData.messageLabel.height + 2;
                }
            }else {
                if (_challengeNoteData.tuningView) {
                    var w:Number = _challengeNoteData.tuningView.width;
                    _challengeNoteData.messageLabel.width = _contentBorder.width - w * 2 / 3;
                    validateLayoutGroup(_challengeNoteData.messageLabel);

                    _challengeNoteData.tuningView.height = _contentBorder.height;
                    validateLayoutGroup(_challengeNoteData.tuningView);

                    _challengeNoteData.tuningView.x = _contentBorder.x + _contentBorder.width - _challengeNoteData.tuningView.width;
                    _challengeNoteData.tuningView.y = _contentBorder.y + (_contentBorder.height - _challengeNoteData.tuningView.height) / 2;
                }
                if (_challengeNoteData.challengeLabel) {
                    _challengeNoteData.challengeLabel.updateView();
                    if (_challengeNoteData.messageLabel) {
                        var w2:Number = _challengeNoteData.challengeLabel.width = 2 * _contentBorder.height;
                        _challengeNoteData.messageLabel.width = _contentBorder.width - w2;
                        _challengeNoteData.challengeLabel.height = actualHeight;
                        _challengeNoteData.challengeLabel.x = _contentBorder.width - w2;
                    }
                }
            }
        }
        if (_challengeNoteData.messageLabel) {
            validateLayoutGroup(_challengeNoteData.messageLabel);
            if(_contentBorder && _dateLabel) {
                _challengeNoteData.messageLabel.y = (actualHeight - (_challengeNoteData.messageLabel.height + _dateLabel.height)) / 2; //Math.min(_contentBorder.y + 20, _dateLabel.y - _challengeNoteData.messageLabel.height);
            }
            _dateLabel.y = _challengeNoteData.messageLabel.y + _challengeNoteData.messageLabel.height + 2;
        }
    }

    private static function validateLayoutGroup(group:DisplayObjectContainer):void{
        for(var i:int = 0; i < group.numChildren; i++){
            var child:DisplayObject = group.getChildAt(i);
            if(child is DisplayObjectContainer){
                validateLayoutGroup(child as DisplayObjectContainer);
            }else if(child.hasOwnProperty("validate")){
                child["validate"]();
            }
            if(group.hasOwnProperty("validate")){
                group["validate"]();
            }
        }
    }

    override protected function commitData():void {
        super.commitData();
        if(!data){
            return;
        }
        if(_challengeNoteData && (_challengeNoteData != data || _challengeNoteData.disposed)){
            while (numChildren) {
                removeChildAt(0);
            }
            removeComponents();
            _challengeNoteData = data  as ChallengeNotificationData;
            _challengeNoteData.holder = this;
            _challengeNoteData.createView();
            invalidate();
        }
        if(!_challengeNoteData) {
            _challengeNoteData = data as ChallengeNotificationData;
            _challengeNoteData.holder = this;
        }

        if(!_bgBottomBorder){
            _bgBottomBorder = new Quad(1, 1, 0x929091);
            _bgBottomBorder.filter = new DropShadowFilter(1, Math.PI);
        }
        if(!_bgBottomBorder.parent){
            addChild(_bgBottomBorder);
        }
        if(!_background) {
            _background = new Quad(1, 1, 0xFFFFFF);
        }
        if(!_background.parent){
            addChildAt(_background, getChildIndex(_bgBottomBorder)+1);
        }
        if(!_icon) {
            _icon = new Image(ButtonFactory.assetManager.getTexture(_challengeNoteData.iconTextureId));
            _icon.scale = _icon.texture.scale/2;
        }
        if(!_icon.parent) {
            addChildAt(_icon, getChildIndex(_background) + 1);
        }
        if(!_contentBorder) {
            _contentBorder = new Image(borderTexture);
            _contentBorder.scale9Grid = new Rectangle(_contentBorder.width / 4, _contentBorder.height / 4, _contentBorder.width / 2, _contentBorder.height / 2);
        }
        if(!_contentBorder.parent){
            addChildAt(_contentBorder, getChildIndex(_icon)+1);
        }

        if(!_dateLabel) {
            _dateLabel = new DateLabel();
            _dateLabel.width = actualWidth / 2;
        }
        if(!_dateLabel.parent){
            addChild(_dateLabel);
        }

        if(_challengeNoteData.messageLabel && !_challengeNoteData.messageLabel.parent) {
            addChild(_challengeNoteData.messageLabel);
        }
        if(_challengeNoteData.tuningView && !_challengeNoteData.tuningView.parent){
            addChild(_challengeNoteData.tuningView);
        }
        if(_challengeNoteData.challengeLabel && !_challengeNoteData.challengeLabel.parent){
            addChild(_challengeNoteData.challengeLabel);
        }
        if(_challengeNoteData.avatar && !_challengeNoteData.avatar.parent){
            addChild(_challengeNoteData.avatar);
        }
        if(_challengeNoteData.buttonGroup && !_challengeNoteData.buttonGroup.parent){
            addChild(_challengeNoteData.buttonGroup);
        }
        _dateLabel.time = _challengeNoteData.time;

        if(data is ChallengeRequestNoteData){
            trace("request");
        }else if(data is ChallengeLostNoteData){
            trace("lost");
        }else if(data is ChallengeWonNoteData){
            trace("won");
        }else if(data is ChallengeRejectedNoteData){
            trace("rejected");
        }else if(data is ChallengeAcceptedNoteData){
            trace("accepted");
        }
        updateView();
        if(_challengeNoteData && _challengeNoteData.disposed){
            _challengeNoteData.createView();
            invalidate();
        }
    }

    override public function dispose():void {
        super.dispose();
        disposeComponents();

        _eventDispatcher = null;

    }

    private function disposeComponents():void{
        if(_icon){
            _icon.removeFromParent(true);
            _icon = null;
        }
        if(_background){
            _background.removeFromParent(true);
            _background = null;
        }
        if(_bgBottomBorder){
            _bgBottomBorder.removeFromParent(true);
            _bgBottomBorder = null;
        }
        if(_contentBorder){
            _contentBorder.removeFromParent(true);
            _contentBorder = null;
        }
        if(_dateLabel){
            _dateLabel.removeFromParent(true);
            _dateLabel = null;
        }
        if(_challengeNoteData){
            _challengeNoteData.dispose();
            _challengeNoteData = null;
        }
    }

    private function removeComponents():void{
        if(_icon){
            _icon.removeFromParent(true);
            _icon = null;
        }
        if(_challengeNoteData){
            _challengeNoteData.remove();
            _challengeNoteData = null;
        }
    }

    public function get eventDispatcher():EventDispatcher {
        return _eventDispatcher;
    }

    public function set eventDispatcher(value:EventDispatcher):void {
        _eventDispatcher = value;
    }

    private static function createBorderTexture():void{
        if(!borderTexture) {
            var bmd:BitmapData = new BitmapData(20, 20, true, 0xFFe6e6e6);
            bmd.fillRect(new Rectangle(1, 1, 18, 18), 0xFFFFFFFF);
            borderTexture = Texture.fromBitmapData(bmd, false, false, 2);
        }
    }
}
}
