package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;

    import starling.display.Quad;
    import starling.utils.ScaleMode;

    public class ChallengeTuningItemRenderer extends ListItemRenderer {
    private var _thumbnail:ImageLoaderWithStates;
    private var _searchResultList:SearchResultList;
    private var loaded:Boolean;
    private var _mask:Quad;
    public function ChallengeTuningItemRenderer() {
        super();
    }

    private function get tuning():Tuning{
        return data as Tuning;
    }

    override protected function commitData():void {
        super.commitData();
        _searchResultList = owner as SearchResultList;
        if(!tuning)
            return;
        if(_thumbnail){
            _thumbnail.removeFromParent(true);
            if(!loaded){
                _searchResultList.loadingItems--;
            }
        }
        _thumbnail = new ImageLoaderWithStates();
        _thumbnail.bitmapScale = 0.25;
        _thumbnail.scaleMode = ScaleMode.NONE;
        _searchResultList.loadingItems++;
        loaded = false;
        var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
        if(tuning.fullHdSnapshots && tuning.fullHdSnapshots[0]){
            simpleImageProvider.urlForDownloading = (tuning.fullHdSnapshots[0].indexOf("http:") == -1?"http:":"") + tuning.fullHdSnapshots[0];
        }else{
            simpleImageProvider.urlForDownloading = tuning.thumbnailURL;
        }
        _thumbnail.persistentImageProvider = simpleImageProvider;
        _thumbnail.completeHandler = function():void{
            var thumbRatio:Number = _thumbnail.image.texture.nativeWidth / _thumbnail.image.texture.nativeHeight;
            var parentRatio:Number = actualWidth / actualHeight;
            if(thumbRatio >= parentRatio){
                _thumbnail.height = actualHeight;
                _thumbnail.width = actualHeight * thumbRatio;
            }else{
                _thumbnail.width = actualWidth;
                _thumbnail.height = actualWidth / thumbRatio;
            }

            _searchResultList.loadingItems = 0;
            loaded = true;
            invalidate("update_size");
        };
        _thumbnail.errorHandler = function():void{
            _searchResultList.loadingItems--;
            loaded = true;
        };

        if(!_mask){
            _mask = new Quad(1,1,0xF1F1F1);
        }

        addChild(_thumbnail);
        addChild(_mask);
        mask = _mask;
    }

    override protected function draw():void{
        super .draw();
        if(isInvalid("update_size")){
            _mask.x = 1;
            _mask.y = 1;
            _mask.width = actualWidth - 2;
            _mask.height = actualHeight - 2;

            _thumbnail.x = (actualWidth - _thumbnail.width) / 2;
            _thumbnail.y = (actualHeight - _thumbnail.height) / 2;

            mask = _mask;
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_thumbnail) {
            _thumbnail.removeFromParent(true);
            _thumbnail = null;
        }
    }
}
}
