package com.despark.tuning3d.view.components.core {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.core.HomeScreenPopUp;

    import feathers.controls.Button;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.PageIndicator;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalLayout;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.filters.DropShadowFilter;
    import starling.textures.Texture;

    public class HomeScreenButton extends LayoutGroup{
    private var _mainImageTexture : Texture;
    private var _topButton:Button;
    private var _tapToChoose:Button;
    private var _wide : Boolean;
    private var _newsList:List;
    private var _pageIndicator:PageIndicator;
    private var _newsItemGroup:LayoutGroup;
    private var _width:Number;
    private var _screen:HomeScreenPopUp;
    private var _screenMode:String;
    private static const RATIO:Number = 0.575; // 0.475
    private static const MIN_DISTANCE:Number = 15;

    public function HomeScreenButton(mainImageTexture : Texture) {
        this._mainImageTexture = mainImageTexture;
        this._wide = Constants.isWide;
        _width = 160;
    }

    override protected function initialize():void {
        super.initialize();
        //
        var vLayout:VerticalLayout = new VerticalLayout();
        vLayout.horizontalAlign = HorizontalAlign.CENTER;
        vLayout.gap = 9;
        layout = vLayout;
        //
        var maxWidth:Number = (stage.stageWidth - 4 * MIN_DISTANCE) / 3;
        if(_width > maxWidth){
            _width = maxWidth;
        }else if(_width / stage.stageHeight < RATIO){
            _width = Math.min(stage.stageHeight * RATIO, maxWidth);
        }
        //
        initTopButton();
        _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);
    }

    private function initTopButton():void {
        var defaultIcon : Image = new Image(_mainImageTexture);
        var downIcon : Image = new Image(_mainImageTexture);
        downIcon.scale = defaultIcon.scale = _width / defaultIcon.width;
        //
        defaultIcon.color = 0xFF0000;
        downIcon.color = 0xFF0000;
        defaultIcon.filter = new DropShadowFilter(0, 0, 0, 0.5, 1.5, 0.32);
        //
        _topButton = new Button();
        _topButton.defaultIcon = defaultIcon;
        _topButton.downIcon = downIcon;
        _topButton.horizontalAlign = HorizontalAlign.CENTER;
        _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);
        //
        var btnSkin:Quad = new Quad(1,1,0xFFFFFF);
        btnSkin.alpha = 0;
        _topButton.defaultSkin = btnSkin;
        width = _width;
        //
        _topButton.width = _width;
        _topButton.height = _width * defaultIcon.height/ defaultIcon.width;
        addChild(_topButton);
        _topButton.validate();
    }

    private function pageChangeHandler(event:Event):void {
        _newsList.removeEventListener(Event.SCROLL, slidesListScrollHandler);
        _newsList.scrollToPageIndex(_pageIndicator.selectedIndex, 0, 0.5);
        restoreScrollHandling();
    }

    private function restoreScrollHandling():void{
        if(_newsList.isScrolling){
            Starling.juggler.delayCall(restoreScrollHandling, 0.01);
        }else if(!_newsList.hasEventListener(Event.SCROLL, slidesListScrollHandler)){
            _newsList.addEventListener(Event.SCROLL, slidesListScrollHandler);
        }
    }

    protected function slidesListScrollHandler(event:Event):void {
        setPageIndicatorPage(_newsList.horizontalPageIndex);
    }

    private function setPageIndicatorPage(value:int):void{
        _pageIndicator.removeEventListener(Event.CHANGE, pageChangeHandler);
        _pageIndicator.selectedIndex = value;
        _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
    }

    private function onButtonSelected(event:Event):void {
        dispatchEventWith(Event.TRIGGERED);
    }


    override public function validate():void {
        if(_topButton){
            _topButton.validate();
        }
        if(_tapToChoose){
            _tapToChoose.validate();
        }
        if(_newsList){
            _newsList.validate();
        }
        if(_pageIndicator){
            _pageIndicator.validate();
        }
        if(_newsItemGroup){
            _newsItemGroup.validate();
        }
        height = NaN;
        super.validate();
    }

    public function set screen(value:HomeScreenPopUp):void {
        _screen = value;
    }

    public function set screenMode(value:String):void {
        _screenMode = value;
    }
}
}
