package com.despark.tuning3d.view.components.gallery
{
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.users.User;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.core.StackedScreen;
	import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.despark.tuning3d.view.components.tunings.TuningsListScreen;
import com.utils.DisplayObjectUtil;

import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalAlign;

import flash.filesystem.File;

	import starling.core.Starling;

	import starling.core.Starling;

import starling.display.Image;

import starling.display.Quad;
import starling.events.Event;
import starling.events.EventDispatcher;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.TextureSmoothing;

public class GalleryTuningItem extends ListItemRenderer {
		private var clipRectQuad:Quad;
		private var backgroundImage:ImageLoaderWithStates=null;
		private var userAvatar:UserCircularAvatar;
		public var count:int = 0;
		public var horizontalAlign:String;
		public var verticalAlign:String;
    	private var _eventDispatcher:EventDispatcher;
		private var _user:User;
		private var imgX:Number;
		private var imgY:Number;
		private var imgW:Number;
		private var imgH:Number;

		public var row:int;
		public var cell:int;
	
		//
		private var carTransform:Object;
		private var _imageWidth:Number = 0;
		private var _imageHeight:Number = 0;
		public function GalleryTuningItem(){

		}

		public function get tuning() : Tuning {
			return data as Tuning;
		}
		
		protected override function initialize() : void {
			autoResize = false;
			clipRectQuad = new Quad(1,1, 0xFFFFFF);
			backgroundImage = new ImageLoaderWithStates();
			backgroundImage.overrideDispose = true;
			backgroundImage.visible=false;
			addChild(backgroundImage);
			addChild(clipRectQuad);

			super.initialize();
			this.label.visible=false;
            addEventListener(TouchEvent.TOUCH, triggeredHandler);
		}

        override public function dispose():void {
            DisplayObjectUtil.disposeMesh(clipRectQuad);
			clipRectQuad = null;
            if(backgroundImage) {
                if(backgroundImage.currentTexture){
                    backgroundImage.currentTexture.dispose();
				}
                backgroundImage.removeFromParent(true);
                backgroundImage = null;
            }

			if(userAvatar){
                userAvatar.removeFromParent(true);
                userAvatar = null;
			}

            removeEventListener(TouchEvent.TOUCH, triggeredHandler);
            super.dispose();
        }

        protected override function draw() : void {
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			if(!backgroundImage.mask){
				if(count == 1){
                    clipRectQuad.width = actualWidth;
                    clipRectQuad.height = actualHeight;
                    clipRectQuad.x = 0;
                    clipRectQuad.y = 0;
				}else{
                    if(horizontalAlign == HorizontalAlign.CENTER){
                        clipRectQuad.width = actualWidth - 2;
                        clipRectQuad.x = 1;
                    }else{
                        clipRectQuad.width = actualWidth - 1;
                        if(horizontalAlign == HorizontalAlign.RIGHT){
                            clipRectQuad.x = 1;
                        }
                    }

                    if(verticalAlign == VerticalAlign.MIDDLE){
                        clipRectQuad.height = actualHeight - 2;
                        clipRectQuad.y = 1;
                    }else{
                        clipRectQuad.height = actualHeight - 1;
                        if(verticalAlign == VerticalAlign.BOTTOM){
                            clipRectQuad.y = 1;
                        }
                    }
				}
                backgroundImage.mask = clipRectQuad;
			}

			if(imageLoader){
				imageLoader.overrideDispose = true;
			}
			
			if(sizeInvalid){
                backgroundImage.x = (actualWidth - backgroundImage.width) / 2;
                backgroundImage.y = (actualHeight - backgroundImage.height) / 2;
			}

			if(imageLoader.image){
                imgX = imageLoader.image.x;
                imgY = imageLoader.image.y;
                imgW = imageLoader.image.width;
                imgH = imageLoader.image.height;
			}
		}
		
		protected function completeLoadImage(imageWidth:Number, imageHeight:Number):void{
			_imageWidth = imageWidth;
			_imageHeight = imageHeight;
			imageLoader.visible = false;
			layout();
			Starling.current.juggler.delayCall(layout, 0.1);
		}

		override protected function layout():void{
			if(!imageLoader){
				return;
			}
			super .layout();
			var bgScale:Number = 1;
			var imgScale:Number;
			if(backgroundImage && backgroundImage.image){
				bgScale = backgroundImage.image.scale;
			}
			if(bgScale == 1){
				bgScale = actualScale;
				Starling.current.juggler.delayCall(layout, 0.1);
				imageLoader.visible = false;
			}else{
				imageLoader.visible = true;
			}
			imgScale = bgScale * carTransform.zoomLevel;
			imageLoader.width = _imageWidth * imgScale;
			imageLoader.height = _imageHeight * imgScale;
			imageLoader.x = (actualWidth - imageLoader.width)/2 + carTransform.pan.x * imgScale + tuningDx;
			imageLoader.y = (actualHeight - imageLoader.height)/2 + carTransform.pan.y * imgScale + tuningDy;
		}

		protected function backgroundCompleteLoadImage(imageWidth:Number, imageHeight:Number):void{
			var imgRatio:Number = imageWidth / imageHeight;
			var itemRatio:Number = actualWidth / actualHeight;
            if(imgRatio < itemRatio) {
                backgroundImage.width = actualWidth;
                backgroundImage.height = actualWidth / imgRatio;
            }else{
                backgroundImage.height = actualHeight;
                backgroundImage.width = actualHeight * imgRatio;
            }
            backgroundImage.x = (actualWidth - backgroundImage.width) / 2;
            backgroundImage.y = (actualHeight - backgroundImage.height) / 2;
		}
		
		protected override function commitData() : void{
			super.commitData();
			var carBackground:CarBackground;
            var cameraIndex:int = 0;
			if(tuning && tuning.carTransform && tuning.carTransform.length){
				carTransform = tuning.carTransform[0];
			}
			carTransform ||= {zoomLevel: 1, pan: {x: 0, y: 0}};
			
			if(tuning && tuning.backgroundId && tuning.backgroundId != CarBackground.DEFAULT_ID && tuning.backgroundsMap && tuning.backgroundsMap[tuning.backgroundId]){
				carBackground= tuning.backgroundsMap[tuning.backgroundId];
                cameraIndex = 0;
				if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0 && carBackground.imagePathSecondCam && carBackground.imagePathSecondCam.length>0){
					cameraIndex = tuning.customSnapshotsCamera[0];
				}
				carBackground.showThumbnailUrl=false;
				carBackground.selectedCamera = cameraIndex;
				backgroundImage.smoothing = TextureSmoothing.BILINEAR;
				backgroundImage.persistentImageProvider = carBackground;
				backgroundImage.visible = true;
			}else if(tuning){
				var randomSplashUrl:String;
				if(tuning && tuning.carChooserBgName) {
                    randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/" + tuning.carChooserBgName.replace('.jpg', '') + ".jpg").url;
                    if (tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length > 0) {
                        cameraIndex = tuning.customSnapshotsCamera[0];
                        if (tuning.carObj.defaultBkgMobile[cameraIndex]) {
                            randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
                        }
                    }
                    carBackground = new CarBackground();
                    carBackground.mainUrl = randomSplashUrl;
                    carBackground.isLocalFile = true;
                    carBackground.showThumbnailUrl = false;
                    backgroundImage.persistentImageProvider = carBackground;
                }else if(tuning && tuning.backgroundId && tuning.backgroundId.indexOf('data.3dtuning.com') != -1){
					if(tuning.backgroundId.indexOf('http:') == -1){
                        tuning.backgroundId = 'http:' + tuning.backgroundId;
					}
                    carBackground = new CarBackground();
                    carBackground.mainUrl = tuning.backgroundId;
                    carBackground.isLocalFile = true;
                    carBackground.showThumbnailUrl = false;
                    backgroundImage.persistentImageProvider = carBackground;
				}else{
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/a_bridge.jpg").url;
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					backgroundImage.persistentImageProvider = carBackground;
				}
				backgroundImage.visible=true;
				backgroundImage.recieveImageSize = backgroundCompleteLoadImage;
			}
			if(tuning){
				var car:Car = new Car();
				car.imageURL = tuning.imageURL;
				imageLoader.recieveImageSize = completeLoadImage;
				if(tuning.userObj){
					_user = new User(tuning.userObj);
					if(count == 1){
                        userAvatar = new UserCircularAvatar(_user, 30, true, 1.5);
						imageLoader.bitmapScale = 1;
					}else if(count == 4){
                        userAvatar = new UserCircularAvatar(_user, 30, true, 1.5);
                        imageLoader.bitmapScale = 0.5;
					}else if(count == 9){
                        userAvatar = new UserCircularAvatar(_user, 25, true, 1.5);
                        imageLoader.bitmapScale = 0.3;
					}
					if(userAvatar){
						userAvatar.addEventListener(TouchEvent.TOUCH, avatarTouchHandler);
						addChild(userAvatar);
						userAvatar.x = actualWidth - userAvatar.width - 7;
                        if(count == 4 && verticalAlign == VerticalAlign.BOTTOM){
                            userAvatar.y = actualHeight - (userAvatar.height + 7);
                        }else{
                            userAvatar.y = 7;
                        }
					}
				}
                imageLoader.persistentImageProvider = car;
			}
		}

	private function avatarTouchHandler(event:TouchEvent):void {
		var touch:Touch = event.getTouch(userAvatar, TouchPhase.ENDED);
		if(touch && (!UsersProxy.loggedInUser() || _user.id != UsersProxy.loggedInUser().id)){
			_eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED, false, {user:_user});
			event.stopImmediatePropagation();
		}
	}

		public function set eventDispatcher(eventDispatcher:EventDispatcher):void {
			_eventDispatcher = eventDispatcher;
		}

    private function triggeredHandler(event:TouchEvent):void {
		var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
		if(_eventDispatcher && touch && !owner.isScrolling){
			SoundManager.playSound(SoundManager.PART_MENU);
			_eventDispatcher.dispatchEvent(new Event(TuningsListScreen.TUNING_SELECTED, false, {user:_user, tuning:tuning}));
            Starling.juggler.delayCall(function():void{
				if(imageLoader.image && imgW && imgH) {
                    imageLoader.image.width = imgW;
                    imageLoader.image.height = imgH;
                    imageLoader.image.x = imgX;
                    imageLoader.image.y = imgY;
                }
            }, .01);
		}
    }

	private function get actualScale():Number{
		if(stage){
			if(actualWidth/actualHeight < stage.stageWidth / stage.stageHeight){
				return actualWidth / stage.stageWidth;
			}
			return actualHeight / stage.stageHeight;
		}
		return 0.3;
	}

	public function get tuningDx():Number{
		var dx:Number = customOffset.left;
		var ds:Number = 0.06 * (ButtonFactory.assetManager.scaleFactor - 1.5) / 2.5 + 0.24;
		if(tuning && tuning.car){
			/*if(!tuning.car.hdsize && tuning.car.carType == "moto" && false){
				ds = (carTransform.zoomLevel - 0.5) * 2 * 0.2 + 0.11;
				dx = 10 * ds;
			}else{*/
				dx *= ds;
			//}
		}
		return dx;
	}

	public function get tuningDy():Number{
		var dy:Number = customOffset.bottom;
		var ds:Number = 0.06 * (ButtonFactory.assetManager.scaleFactor - 1.5) / 2.5 + 0.24;
		return dy * ds + (0.73 * actualScale - 0.27) * dy;
	}

	private function get customOffset():Object{
		if(tuning){
			if(tuning.carObj && tuning.carObj.custom_offset){
				return tuning.carObj.custom_offset;
			}
			if(tuning.car && tuning.car.custom_offset){
				return tuning.car.custom_offset
			}
			return {bottom:0, left: 0};
		}
		return {bottom:0, left: 0};
	}

}
}
