package com.despark.tuning3d.view.components.tunings.colors
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.Part;
	import com.despark.tuning3d.model.parts.PartDisk;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.parts.PartSlotDisk;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.parts.PartDiskItemRenderer;
	import com.despark.tuning3d.view.components.tunings.editing.ValueStepper;
	import com.utils.ObjectUtils;

	import feathers.controls.Button;
	import feathers.controls.LayoutGroup;
	import feathers.controls.List;
	import feathers.controls.ScrollContainer;
	import feathers.core.FeathersControl;
	import feathers.layout.HorizontalAlign;
	import feathers.layout.VerticalAlign;
	import feathers.layout.VerticalLayout;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class TuningDiskSettingsView extends FeathersControl implements Multilingual {
		public static const CLOSE_DISK_SETTINGS:String="CloseDisksSettingsMenu";
		
		public static const SHOW_BUY_PACKET_CAMBER:String="SHOW_BUY_PACKET";
		
	
		public static const ITEM_WIDTH:Number=20;
		public static const ITEM_HEIGHT:Number=20;

		public var assetManager : AssetManager;

		private var currentLanguage : Language;
		private var quad : Quad;
		private var _partSlot : PartSlot;

		private var curRadius : int = 15;
		private var _min_size:int=0;
		private var _max_size:int=0;
		private var btnClose:Button;


		private var _rotateDataProvider:Array;
		private var _radiusDataProvider:Array;
		private var _camberDataProvider:Array;
		private var _offsetDataProvider:Array;
		private var _profileDataProvider:Array;

		private var spinner:Sprite;
		private var blocker:Quad;
		private var buttonLock:Button;
		protected var carLock:Image;

        private static const PART_GAP:Number = 15;

		private var _owner:List;

		private var steeringStepper:ValueStepper
		private var radiusStepper:ValueStepper
		private var offsetStepper:ValueStepper
		private var camberStepper:ValueStepper
		private var profileStepper:ValueStepper
		private var delimiterList:Vector.<Quad>;


		public function TuningDiskSettingsView() {
			super();
		}

		private function isExistsState(id:String):Boolean{
			if(!_partSlot)
				return false;
			if(!_partSlot.selectedPart)
				return false;
			var part:PartDisk=_partSlot.selectedPart as PartDisk;
			if(!part)
				return false;
			for (var nameOfState:String in part.states){
				if(nameOfState.indexOf(id)>=0){
					return true;
				}
			}
			return false;
		}
		
		public function set partSlot(value : PartSlot) : void {
			_partSlot = value;
			if(_partSlot){
				var part:PartDisk=_partSlot.selectedPart as PartDisk;
				var partDisk:PartDisk=part;

				_rotateDataProvider = [];
				if(isTrlPossible()){
					_rotateDataProvider.push({
						value: "trl",
						title: "Left"
					});
				}
				_rotateDataProvider.push({
					value: null,
					title: "Straight"
				});
				if(isTrrPossible()){
					_rotateDataProvider.push({
						value: "trr",
						title: "Right"
					});
				}

				_camberDataProvider = [];
				_camberDataProvider.push({
					id: null,
					title: "STOCK"
				});

				if(isExistsState("cm1")){
					_camberDataProvider.push({     
						id:"cm1",
						title: "CUSTOM"
					});
				}
				
				if(isExistsState("cm2")){
					_camberDataProvider.push({
						id:"cm2",
						title: "EXTREME"
					});
				}

				_offsetDataProvider = [];
				_offsetDataProvider.push({
					id: null,
					title:"STOCK"
				});

				if(isExistsState("of1")){
					_offsetDataProvider.push({
						id:"of1",
						title:"CUSTOM"
					});
				}
					
				if(isExistsState("of2")){
					_offsetDataProvider.push({
						id: "of2",
						title:"EXTREME"
					});
				}

				if(partDisk && partDisk.radius > 0){
					_min_size = part.properties.min_size;
					_max_size = part.properties.max_size;
					if(part.selectedDiskSize > 0){
						curRadius = part.selectedDiskSize;
					}else{
						curRadius = part.properties.size;
					}
					_radiusDataProvider = [];
					var rList:Array = PartSlotDisk(_partSlot).radiusList;
					rList.sort(Array.NUMERIC);
					for(var i:int = 0; i < rList.length; i++){
						_radiusDataProvider.push({
							id: "R" + rList[i].toString(),
							value: rList[i],
							title:"R" + rList[i].toString()
						});
					}
				}else{
					if(part.selectedDiskSize > 0){
						curRadius = part.selectedDiskSize;
					}else{
						curRadius = part.properties.size;
					}
					_radiusDataProvider = [{
						id: "R" + curRadius.toString(),
						value: curRadius,
						title:"R" + curRadius.toString()
					}];
				}

				_profileDataProvider = [];
                if(hasProfile(_partSlot)){
					_profileDataProvider.push({
						id:"FLAT",
						title:"FLAT"
					},{
						id:"MID",
						title:"MID"
					},{
						id:"DEEP",
						title:"DEEP"
					},{
						id:"S.DEEP",
						title:"S.DEEP"
					});
                }else{
					_profileDataProvider.push({
						id:"D",
						title:"DEFAULT"
					});
				}

				steeringStepper.data = _rotateDataProvider;
				radiusStepper.data = _radiusDataProvider;
				for(i = 0; i < _radiusDataProvider.length; i++){
					if(_radiusDataProvider[i].value == curRadius){
						setRadiusCurrentValue(i);
						break;
					}
				}

				offsetStepper.data = _offsetDataProvider;
				camberStepper.data = _camberDataProvider;
				profileStepper.data = _profileDataProvider;

				invalidate("partSlot");
			}
		}

		private function delimiterLayout():void {
		}

        private function hasProfile(ps:PartSlot):Boolean {
			var diskSlot:PartSlotDisk = ps as PartSlotDisk;
			if(!diskSlot){
				return false;
			}
			var part:PartDisk = diskSlot.selectedPart as PartDisk;
			return part.profile != 'default';
        }
		
		public function get partSlot() : PartSlot {
			return _partSlot;
		}

		public function updateWithLanguage(language : Language) : void {
			currentLanguage = language;
		}

		override protected function initialize() : void {
			super.initialize();
			quad = new Quad(1, 1, 0x232325);
			quad.alpha=1;
			addChild(quad);
			
			btnClose = new Button();
			btnClose.addEventListener(Event.TRIGGERED, closeViewHandler);
			btnClose.labelFactory = LabelFactory.valueStepperTitle;
			var closeDef:Image = new Image(ButtonFactory.assetManager.getTexture("a_close_effect_menu"));
			closeDef.scale = 1.36 * ButtonFactory.assetManager.scaleFactor / 2;
			btnClose.defaultIcon = closeDef;

			closeDef = new Image(ButtonFactory.assetManager.getTexture("a_close_effect_menu"));
			closeDef.scale = 1.36 * ButtonFactory.assetManager.scaleFactor / 2;
			closeDef.alpha = 0.6;
			btnClose.downIcon = closeDef;
			btnClose.disabledIcon = closeDef;
			btnClose.horizontalAlign = HorizontalAlign.CENTER;
			btnClose.verticalAlign = VerticalAlign.MIDDLE;
			addChild(btnClose);
			btnClose.label = "Close";
			btnClose.iconOffsetY = -2;
			btnClose.width = 35;
			btnClose.height = 50;

			blocker=new Quad(1,1,0x000000);
			blocker.visible = false;
			blocker.alpha = 0.3;
			addChild(blocker);

			spinner=new Sprite();
			spinner.visible=false;
			
			var spinnerTexture:Texture=assetManager.getTexture("spinner");
			var spinnerImage:Image=new Image(spinnerTexture);
			spinnerImage.width = 16;
			spinnerImage.height = 16;
			spinnerImage.x =- 8;
			spinnerImage.y =- 8;
			
			spinner.addChild(spinnerImage);
			//addChild(spinner);
			//addEventListener(EnterFrameEvent.ENTER_FRAME, enterFrameHandler);

			steeringStepper = new ValueStepper();
			steeringStepper.setIcons("stepper_left", "stepper_right");
			steeringStepper.width = 90;
			steeringStepper.height = 30;
			steeringStepper.y = 15;
			steeringStepper.addEventListener(Event.CHANGE, steeringStepperChangeHandler);
			addChild(steeringStepper);
			steeringStepper.title = "Steering wheel";

			radiusStepper = new ValueStepper();
			radiusStepper.setIcons("stepper_minus", "stepper_plus");
			radiusStepper.width = 100;
			radiusStepper.height = 30;
			radiusStepper.y = 15;
			radiusStepper.addEventListener(Event.CHANGE, radiusStepperChangeHandler);
			addChild(radiusStepper);
			radiusStepper.title = "Diameter";
			radiusStepper.border = true;

			offsetStepper = new ValueStepper();
			offsetStepper.setIcons("stepper_minus", "stepper_plus");
			offsetStepper.width = 100;
			offsetStepper.height = 30;
			offsetStepper.y = 15;
			offsetStepper.addEventListener(Event.CHANGE, offsetListChangeHandler);
			addChild(offsetStepper);
			offsetStepper.title = "Offset";

			camberStepper = new ValueStepper();
			camberStepper.setIcons("stepper_minus", "stepper_plus");
			camberStepper.width = 100;
			camberStepper.height = 30;
			camberStepper.y = 15;
			camberStepper.addEventListener(Event.CHANGE, camberListChangeHandler);
			addChild(camberStepper);
			camberStepper.title = "Camber";

			profileStepper = new ValueStepper();
			profileStepper.setIcons("stepper_minus", "stepper_plus");
			profileStepper.width = 100;
			profileStepper.height = 30;
			profileStepper.y = 15;
			profileStepper.addEventListener(Event.CHANGE, profileListChangeHandler);
			addChild(profileStepper);
			profileStepper.title = "Profile";

			var i:int;
			delimiterList = new Vector.<Quad>();
			delimiterList.push(createDelimiter(50));
			for(i = 0; i < 4; i++){
				delimiterList.push(createDelimiter());
			}

			for(i = 0; i < delimiterList.length; i++){
				addChild(delimiterList[i]);
			}


			buttonLock = new Button();
			buttonLock.visible = false;

			buttonLock.defaultSkin = new Quad(10,10,0x000000);
			buttonLock.defaultSkin.alpha = 0.3;
			buttonLock.addEventListener(Event.TRIGGERED, onClickBlockerLock);
			addChild(buttonLock);

			var carLockTexture : Texture = AlertView.assetManager.getTexture("car_lock_big_icon");
			carLock=new Image(carLockTexture);
			carLock.visible=false;
			carLock.touchable=false;
			carLock.addEventListener(Event.TRIGGERED, onClickBlockerLock);
			addChild(carLock);
		}

		private function setSteeringCurrentValue(value:int):void{
			steeringStepper.removeEventListener(Event.CHANGE, steeringStepperChangeHandler);
			steeringStepper.value = value;
			steeringStepper.addEventListener(Event.CHANGE, steeringStepperChangeHandler);
		}

		private function setRadiusCurrentValue(value:int):void{
			radiusStepper.removeEventListener(Event.CHANGE, radiusStepperChangeHandler);
			radiusStepper.value = value;
			radiusStepper.addEventListener(Event.CHANGE, radiusStepperChangeHandler);
		}

		private function setCamberCurrentValue(value:int):void{
			camberStepper.removeEventListener(Event.CHANGE, camberListChangeHandler);
			camberStepper.value = value;
			camberStepper.addEventListener(Event.CHANGE, camberListChangeHandler);
		}

		private function setOffsetCurrentValue(value:int):void{
			offsetStepper.removeEventListener(Event.CHANGE, offsetListChangeHandler);
			offsetStepper.value = value;
			offsetStepper.addEventListener(Event.CHANGE, offsetListChangeHandler);
		}

		private function setProfileCurrentValue(value:int):void{
			profileStepper.removeEventListener(Event.CHANGE, profileListChangeHandler);
			profileStepper.value = value;
			profileStepper.addEventListener(Event.CHANGE, profileListChangeHandler);
		}

		private function steeringStepperChangeHandler(event:Event):void{
			SoundManager.playSound(SoundManager.SELECT_PART);
			if(steeringStepper.selectedItem.value){
				doChangeData(false, false);
			}else{
				doChangeData(false, true);
			}
		}
		
		public function get owner():List{
			return _owner;
		}

		public function set owner(value:List):void {
			_owner = value;
		}
		
		private function radiusStepperChangeHandler(event:Event = null):void {
			var delta:int = radiusStepper.value - radiusStepper.prevValue;
			changeRadius(delta);
			if(owner.selectedItem){
				var partDisk:PartDisk = (owner.selectedItem as  PartDisk);
				partDisk.selectedDiskSize = curRadius;
				var profileValue:String = PartDisk(partDisk.parentSlotDisk.selectedPart).profile;
				var prefix:String=PartSlotDisk.getPrefixDep(partDisk);
				var partDiskObj:Object = partDisk.parentSlotDisk.findPartDiskForRadius(partDisk.name+prefix, partDisk.selectedDiskSize, profileValue);
				while(!partDiskObj && curRadius > _min_size && curRadius < _max_size){
					changeRadius(delta);
					partDisk.selectedDiskSize = curRadius;
					prefix = PartSlotDisk.getPrefixDep(partDisk);
					partDiskObj = partDisk.parentSlotDisk.findPartDiskForRadius(partDisk.name+prefix, partDisk.selectedDiskSize, profileValue);
				}
				SoundManager.playSound(SoundManager.SELECT_PART_BACK);
				owner.dispatchEventWith(PartDiskItemRenderer.EVENT_CHANGE_PART_DATA, false, {disk: partDiskObj});
			}
		}

		private function changeRadius(delta:int):void {
			curRadius += delta;
		}
		
		private function onClickBlockerLock(event:Event):void{
			SoundManager.playSound(SoundManager.DISABLED_PART);
			dispatchEventWith(SHOW_BUY_PACKET_CAMBER, true);
		}
		
		
		private function enterFrameHandler(event : EnterFrameEvent) : void {
			if(spinner && spinner.visible){
				spinner.rotation += 0.2;
			}
		}

		public function doChangeData(neon_on:Boolean=false, unselect:Boolean=false):void{
			if(!sendDataChangeEvent)
				return;
			var part:PartDisk=_partSlot.selectedPart as PartDisk;
			part.parentSlotDisk.selectedPart = part;
			if(part && part.selectedState && part.selectedState.nameOfState && part.selectedState.nameOfState.indexOf("neon")>=0){
				neon_on=true;
			}    
			
			var stateName:String="";
			if(camberStepper.value){
				stateName = camberStepper.selectedItem.id;
			}
			
			if(neon_on){
				if(stateName!=""){
					stateName+="-";
				}
				stateName+="neon";
			}
			
			if(offsetStepper.value){
				if(stateName!=""){
					stateName+="-";
				}
				stateName += offsetStepper.selectedItem.id;
			}

			if(steeringStepper.selectedItem.value){
				if(stateName != ""){
					stateName += "-";
				}
				stateName += steeringStepper.selectedItem.value;
			}

			for(var nameOfState:String in part.states){
				if(nameOfState==stateName){
					enableButtons(false);
					dispatchEventWith(TuningSelectedStateView.STATE_SELECTED, true, {selectedState: nameOfState, part: Part});
					return;
				}
			}
			
			nameOfState=null;
			if(neon_on){
				nameOfState="neon";
			}
			
			if(unselect){
				enableButtons(false);
				dispatchEventWith(TuningSelectedStateView.STATE_SELECTED, true, {selectedState: nameOfState, part: Part});
			}
		}
		
		public function enableButtons(value:Boolean):void{
			this.blocker.visible =! value;
			this.spinner.visible =! value;
			touchable = value;
		}

		private function camberListItemSelectedAgain(event:Event):void{
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			doChangeData(false, true);
		}
		
		private function camberListChangeHandler(event:Event):void{
			if(camberStepper.value == 0){
				camberListItemSelectedAgain(null);
				return;
			}
			SoundManager.playSound(SoundManager.SELECT_PART);
			doChangeData();
		}

		private function offsetListItemSelectedAgain(event:Event):void{
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			doChangeData(false, true);
		}
		
		private function offsetListChangeHandler(event:Event):void{
			if(offsetStepper.value == 0){
				offsetListItemSelectedAgain(null);
				return;
			}
			SoundManager.playSound(SoundManager.SELECT_PART);
			doChangeData();
		}

		private function profileListChangeHandler(event:Event):void {
            var profileValue:String = profileStepper.selectedItem ? profileStepper.selectedItem.id : null;
            var partDisk:PartDisk = PartDisk(partSlot.selectedPart)||(owner.selectedItem as PartDisk);
            if (partDisk) {
				partDisk.selectedDiskSize = curRadius;
				var prefix:String = PartSlotDisk.getPrefixDep(partDisk);
				var partDiskObj:Object = partDisk.parentSlotDisk.findPartDiskForRadius(partDisk.name + prefix, partDisk.selectedDiskSize, profileValue);
				SoundManager.playSound(SoundManager.SELECT_PART);
				owner.dispatchEventWith(PartDiskItemRenderer.EVENT_CHANGE_PART_DATA, false, {disk: partDiskObj});
			}
		}
		
		public function removeCamberLock():void{
			trace("removeCamberLock");
			buttonLock.visible = false;
			carLock.visible = false;
			if(_partSlot && _partSlot.selectedPart){
				trace("removeCamberLock2");
				_partSlot.selectedPart.camberLock = false;
				invalidate();
			}
		}
		
		override protected function draw() : void {
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			var i:int;
			if(isInvalid("partSlot")){
				var stateName:String = "";
				if(partSlot.selectedPart && partSlot.selectedPart.selectedState && partSlot.selectedPart.selectedState.nameOfState){
					stateName = partSlot.selectedPart.selectedState.nameOfState;
				}
				
				sendDataChangeEvent = false;

				var rotVal:String;
				if(stateName.indexOf("trr") >= 0){
					rotVal = "trr";
				}else if(stateName.indexOf("trl") >= 0){
					rotVal = "trl";
				}
				for(i = 0; i < _rotateDataProvider.length; i++){
					if(_rotateDataProvider[i].value == rotVal){
						setSteeringCurrentValue(i);
						break;
					}
				}

				if(stateName.indexOf("of1") >= 0){
					setOffsetCurrentValue(1);
                }else if(stateName.indexOf("of2") >= 0){
					setOffsetCurrentValue(2);
                }else{
					setOffsetCurrentValue(0);
				}

				if(stateName.indexOf("cm1") >= 0){
					setCamberCurrentValue(1);
                }else if(stateName.indexOf("cm2") >= 0){
					setCamberCurrentValue(2);
                }else{
					setCamberCurrentValue(0);
				}

				if(_partSlot && hasProfile(_partSlot)) {
					var profileValues:Array = ObjectUtils.getKeys(PartSlotDisk(_partSlot).diskRadiusMap[partSlot.selectedPart.name][PartDisk(partSlot.selectedPart).radius], false);
                    var dat:Object;
					for(i = _profileDataProvider.length-1; i >= 0; i--){
                        dat = _profileDataProvider[i];
                        if(profileValues.indexOf(dat.id)==-1){
                            _profileDataProvider.removeAt(i);
                        }
                    }
                    for(i = _profileDataProvider.length-1; i >= 0; i--){
                        dat = _profileDataProvider[i];
                        if(dat.id == PartDisk(partSlot.selectedPart).profile){
                            setProfileCurrentValue(i);
							break;
                        }
                    }
                }

				sendDataChangeEvent=true;
				sizeInvalid=true;
				if(_partSlot && _partSlot.selectedPart){
					var part:PartDisk=_partSlot.selectedPart as PartDisk;
					if (part) {
						part.camberLock = PartDisk(_partSlot.possibleParts[0]).camberLock;
						if (part.camberLock) {
							buttonLock.visible = true;
							carLock.visible = true;
						} else {
							buttonLock.visible = false;
							carLock.visible = false;
						}
					}
				}

				changeRadius(0);
			}
			
			
			if(sizeInvalid) {
				if(quad) {
					quad.width = actualWidth;
					quad.height = actualHeight;
				}

				if(btnClose){
					//btnClose.labelOffsetX = -btnClose.defaultIcon.width;
					btnClose.labelOffsetY = btnClose.defaultIcon.height + 3;
					btnClose.validate();
					btnClose.labelOffsetX = -btnClose.defaultIcon.width / 2 - (btnClose.width - btnClose.defaultIcon.width) / 2 - 1;
					btnClose.x = -btnClose.labelOffsetX - 3;
					btnClose.y = (actualHeight - btnClose.height)/2 - 3;
				}


				var closeSize:Number = btnClose.width + 10;
				var stepperList:Array = [steeringStepper, radiusStepper, offsetStepper, camberStepper, profileStepper];
				var stepSize:Number = (actualWidth - closeSize) / stepperList.length;
				for(i = 0; i < stepperList.length; i++){
					stepperList[i].x = closeSize + i * stepSize + (stepSize - stepperList[i].width) / 2;
					delimiterList[i].x = closeSize + i * stepSize - 0.5;
					delimiterList[i].y = (50 - delimiterList[i].height) / 2;
				}

				if(blocker){
					blocker.width = actualWidth;
					blocker.height = actualHeight;
				}
				
				if(buttonLock){
					buttonLock.width = actualWidth - closeSize;
					buttonLock.height = actualHeight;
					buttonLock.x = closeSize;
				}
				
				if(carLock){
					carLock.x = (actualWidth - carLock.width + closeSize)/2;
					carLock.y = (actualHeight - carLock.height)/2;
				}
				
				if(spinner){
					spinner.x = actualWidth / 2;
					spinner.y = actualHeight / 2;
				}
			}

            delimiterLayout();
		}

        private function isTrrPossible():Boolean{
			for (var nameOfState:String in partSlot.selectedPart.states){
				if(nameOfState && nameOfState.indexOf("trr")>=0) return true;
			}
			return false;
		}
		
		private function isTrlPossible():Boolean{
			for (var nameOfState:String in partSlot.selectedPart.states){
				if(nameOfState && nameOfState.indexOf("trl")>=0) return true;
			}
			return false;
		}

		private static function createDelimiter(borderHeight:Number = -1):Quad{
			var line:Quad = new Quad(1,1, 0xEEEEEE);
			line.width = 1;
			line.height = borderHeight==-1?40:borderHeight;
			return line;
		}

		private var sendDataChangeEvent:Boolean=true;
		
		private function closeViewHandler(event : Event) : void {
			dispatchEventWith(CLOSE_DISK_SETTINGS);
		}
	}
}
