package com.despark.tuning3d.view.components.tunings.editing {
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.NumericStepper;
    import feathers.controls.TextInput;
    import feathers.layout.VerticalAlign;

    import starling.display.Image;
    public class ValueStepper extends NumericStepper {
        private var decrementIcon:String;
        private var incrementIcon:String;
        private var _prevValue:Number;
        private var _data:Array;
        private var _titleField:Label;
        private var borderImage:Image;
        public function ValueStepper() {
            super();

            textInputFactory = function():TextInput {
                var input:TextInput = new TextInput();
                input.textEditorFactory = LabelFactory.valueStepperLabel;
                input.verticalAlign = VerticalAlign.MIDDLE;
                input.isEditable = false;
                input.isSelectable = false;
                input.paddingLeft = 2;
                input.paddingTop = -1;
                input.paddingBottom = 1;
                return input;
            }

            decrementButtonFactory = function():Button {
                var button:Button;
                if(decrementIcon){
                    button = createButton(decrementIcon);
                }else{
                    button = createButton();
                }
                return button;
            }

            incrementButtonFactory = function():Button {
                var button:Button;
                if(incrementIcon){
                    button = createButton(incrementIcon);
                }else{
                    button = createButton();
                }
                return button;
            }

            valueFormatFunction = function(value:Number):String{
                if(_data && _data.length){
                    return _data[value].title;
                }
                return "EMPTY";
            }
        }

        public function set title(str:String):void{
            if(_titleField == null){
                _titleField = new Label();
                _titleField.width = actualWidth;
                _titleField.textRendererFactory = LabelFactory.valueStepperTitle;
                addChild(_titleField);
            }
            _titleField.text = str;
            _titleField.y = -_titleField.height - 10;
        }

        private static function createButton(textureId:String = null):Button{
            var button:Button = new Button();
            if(textureId){
                var defaultSkin:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
                defaultSkin.scale = ButtonFactory.assetManager.scaleFactor / 2;
                var downSkin:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
                downSkin.scale = ButtonFactory.assetManager.scaleFactor / 2;
                downSkin.alpha = 0.2;
                button.defaultSkin = defaultSkin;
                button.downSkin = button.disabledSkin = downSkin;
                button.setSize(defaultSkin.width, defaultSkin.height);
            }
            return button;
        }

        override protected function initialize():void{
            super .initialize();
            createTextInput();
            textInputGap = -10;
            createDecrementButton();
            createIncrementButton();
            minimum = 0;
            step = 1;
        }

        public function set border(val:Boolean):void{
            if(val){
                if(!borderImage){
                    borderImage = new Image(ButtonFactory.assetManager.getTexture("stepper_border"));
                    borderImage.scale = ButtonFactory.assetManager.scaleFactor / 2;
                    addChildAt(borderImage, 0);
                }
            }else{
                if(borderImage){
                    borderImage.removeFromParent(true);
                    borderImage = null;
                }
            }
        }

        public function setIcons(decrementIcon:String, incrementIcon:String):void{
            this.decrementIcon = decrementIcon;
            this.incrementIcon = incrementIcon;
        }

        override protected function draw():void{
            super .draw();
            incrementButton.isEnabled = incrementButton.touchable = !(value == maximum);
            decrementButton.isEnabled = decrementButton.touchable = !(value == minimum);
            if(incrementButton.isEnabled || decrementButton.isEnabled){
                textInput.alpha = 1;
                textInput.touchable = true;
            }else{
                textInput.alpha = 0.2;
                textInput.touchable = false;
            }

            if(borderImage){
                borderImage.x = (actualWidth - borderImage.width) / 2;
                borderImage.y = (actualHeight - borderImage.height) / 2;
            }
        }

        override protected function increment():void{
            if(_prevValue != value){
                _prevValue = value;
            }
            super.increment();
        }

        override protected function decrement():void{
            if(_prevValue != value){
                _prevValue = value;
            }
            super.decrement();
        }

        public function set data(data:Array):void {
            _data = data;
            if(_data && _data.length){
                maximum = _data.length - 1;
                invalidate();
            }
        }

        public function get data():Array {
            return _data;
        }

        public function getItemAt(index:Number):*{
            if(_data && _data[index]){
                return _data[index];
            }
            return null;
        }

        public function get selectedItem():*{
            if(_data && _data[value]){
                return _data[value];
            }
            return null;
        }

        public function get prevValue():Number {
            return _prevValue;
        }
    }
}
