package com.despark.tuning3d.view.core {
    import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.AlertViewHL;
    import com.despark.tuning3d.view.components.core.BlockerView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;
    import com.despark.tuning3d.view.components.core.HomeScreenButton;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.MainMenuItem;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.news.NewsFeedData;
    import com.utils.DisplayObjectUtil;
    import com.utils.FileUtil;

    import feathers.controls.Button;
    import feathers.controls.LayoutGroup;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.filesystem.File;

    import flash.system.Capabilities;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.display.Sprite;
    import starling.events.Event;

    public class HomeScreenPopUp extends StackedScreen implements IMainMenuHolder{

    public static const MODE_SELECTED : String = "ModeSelected";
    public static const RESTORE_SELECTED : String = "HomeScreenPopUpRestoreSelected";

    public static var UPDATE_OBJECT : Object;

    private var _tuningButton : HomeScreenButton;
    private var _truckConfiguratorButton : HomeScreenButton;
    private var _muscleCarsButton : HomeScreenButton;
    private var _mainMenuView:MainMenuView;
    private var _blockerView : BlockerView;


    private var _waitingForBrandsLoad : Boolean = false;
    private var _headerContainer:LayoutGroup;
    private var _wide : Boolean; // true if the device is rectangular ( iphone 7(aspectRation > 1.5)) false if square (iPad)
    private var _isSuperWide : Boolean; // true if the device is too wide( (aspectRation > 1.8)) false if square (iPad)


    private var _selectedLanguage:Language;
    private var _languages:Array;
    private var _language:Language = null;
    private var _userSignedIn:Boolean;
    private var _selectedTuning:Tuning;

    //

    private var galleryButton:Sprite;
    private var garageButton:Sprite;
    private var timelineButton:Sprite;
    private var challengesButton:Sprite;
    private var scheduleButton:Sprite;

    private var mailButton:Sprite;
    private var friendsButton:Sprite;
    private var notificationsButton:Sprite;

    private var loginButton:Sprite;
    private var settingsButton:Button;
    private var newsButton:MainMenuItem;

    public function HomeScreenPopUp() {
        super("HomeScreenPopUp");
        AppMode.self.setModeHome();
    }

    private function initBg() : void {
        var newBg : Image = new Image(assetManager.getTexture("firstScreenBg"));
        var aspect : Number = newBg.width / newBg.height;
        var currentAspect : Number = stage.stageWidth / stage.stageHeight;
        if(currentAspect > aspect) {
            newBg.width = stage.stageWidth;
            newBg.height = stage.stageWidth / aspect;
        } else {
            newBg.height = stage.stageHeight;
            newBg.width = stage.stageHeight * aspect;
        }
        newBg.x = (stage.stageWidth - newBg.width)/2;
        newBg.y = (stage.stageHeight - newBg.height)/2;
        addChild(newBg);
    }


    override protected function initialize():void {
        super.initialize();

        _wide = this.stage.stageWidth / this.stage.stageHeight > 1.5;
        _isSuperWide = this.stage.stageWidth / this.stage.stageHeight > 1.8;

        setSizeInternal(this.stage.stageWidth, this.stage.stageHeight, false);
        initBg();
        initHeader();

        _tuningButton = ButtonFactory.homeScreenButton("tuning");
        _tuningButton.screen = this;
        _tuningButton.screenMode = AppMode.MODE_TUNING;
        _tuningButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_tuningButton);

        _truckConfiguratorButton = ButtonFactory.homeScreenButton("truck");
        _truckConfiguratorButton.screen = this;
        _truckConfiguratorButton.screenMode = AppMode.MODE_TRUCK_CONFIGURATOR;
        _truckConfiguratorButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_truckConfiguratorButton);

        _muscleCarsButton = ButtonFactory.homeScreenButton("muscle");
        _muscleCarsButton.screen = this;
        _muscleCarsButton.screenMode = AppMode.MODE_AMERICAN_MUSCLE;
        _muscleCarsButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_muscleCarsButton);

        locateMainButtons();

        if (!_mainMenuView){
            mainMenuView = new MainMenuView();
            mainMenuView.name = "mainMenuView";
            addChild(_mainMenuView);
            _mainMenuView.visible = false;

            galleryButton = _mainMenuView.galleryButton;
            garageButton = _mainMenuView.grgeButton;
            timelineButton = _mainMenuView.timelineButton
            challengesButton = _mainMenuView.challengeButton;
            scheduleButton = _mainMenuView.shedButton;


            if(_mainMenuView.avatarContainer){
                loginButton = _mainMenuView.avatarContainer;
            }
            settingsButton = ButtonFactory.circleButton("profile");
            settingsButton.setSize(28, 28);
            settingsButton.addEventListener(Event.TRIGGERED, mainMenuView.profileButtonTriggeredHandler);
            newsButton = new MainMenuItem("news_btn", null, true);
            newsButton.addEventListener(Event.TRIGGERED, mainMenuView.newsButtonTriggeredHandler);

            mailButton = _mainMenuView.mailsButton;
            friendsButton = _mainMenuView.frndButton;
            notificationsButton = _mainMenuView.challengeNotesButton;

            addChild(galleryButton);
            addChild(garageButton);
            addChild(timelineButton);
            addChild(challengesButton);
            addChild(scheduleButton);

            if(loginButton){
                addChild(loginButton);
            }
            addChild(settingsButton);
            addChild(newsButton);
            newsButton.noteCount = 0;

            addChild(mailButton);
            addChild(friendsButton);
            addChild(notificationsButton);

            _mainMenuView.removeFromParent();
            addEventListener(RESTORE_SELECTED, restoreEventHandler);
        }

        updateNotificationCount();
        Starling.current.juggler.delayCall(function():void{
            invalidate(INVALIDATION_FLAG_SIZE);
        }, 0.1);

    }

        private function restoreEventHandler(event:Event):void {
            mainMenuView.restoreButtonTriggeredHandler(null);
        }

    private function locateMainButtons():void{
        if(stage) {
            var buttonsWidth:Number = (_tuningButton.width + _truckConfiguratorButton.width + _muscleCarsButton.width);
            var btns:Array = [_tuningButton, _truckConfiguratorButton, _muscleCarsButton];
            var btnGap:Number = (stage.stageWidth - buttonsWidth) / (btns.length + 1);
            var lastX:Number = btnGap;
            for(var i:int = 0; i < btns.length; i++){
                btns[i].x = lastX;
                lastX = btns[i].x + btns[i].width + btnGap;
            }
        }
    }

    public function updateNotificationCount() :void{
        if(mainMenuView){
            mainMenuView.updateNotificationCount();
        }
        if(UPDATE_OBJECT){
            var newsCacheFile:File = File.applicationStorageDirectory.resolvePath("news_feed.json");
            var newsCacheContent:Object;
            if(newsCacheFile.exists){
                newsCacheContent = JSON.parse(FileUtil.fileGetContent(newsCacheFile));
            }else{
                newsCacheContent = {};
            }
            var newsCount:int = 0;
            for(var feedType:String in UPDATE_OBJECT){
                if(NewsFeedData.isAllowedType(feedType)){
                    for(var i:int = 0; i < UPDATE_OBJECT[feedType].length; i++){
                        var feedId:String = UPDATE_OBJECT[feedType][i]._id;
                        if(!newsCacheContent.hasOwnProperty(feedId)){
                            newsCacheContent[feedId] = true;
                            newsCount ++;
                        }
                    }
                }
            }
            if(newsButton){
                newsButton.noteCount = newsCount;
            }
        }
    }

    override protected function draw():void {
        super.draw();

        if(_headerContainer && _tuningButton && _truckConfiguratorButton && _muscleCarsButton) {
            var yCoord : Number ;
            _headerContainer.validate();
            _headerContainer.validate();
            _tuningButton.validate();
            //
            locateMainButtons();
            if(_tuningButton.height == 0 || _truckConfiguratorButton.height == 0 || _muscleCarsButton.height == 0){
                Starling.juggler.delayCall(invalidate, .25);
                return;
            }
            //
            yCoord = (stage.stageHeight - _tuningButton.height) / 2;
            _tuningButton.y = yCoord;
            _truckConfiguratorButton.y = yCoord;
            _muscleCarsButton.y = yCoord;
            var btn:Sprite;
            var leftGroup:Array = [loginButton, settingsButton, newsButton];
            var topBtnMargin:Number = 15;
            try{
                var screenRatio:Number = Starling.current.nativeStage.stage.fullScreenWidth / Capabilities.screenDPI;
                if(screenRatio < 3.5){
                    topBtnMargin = 8;
                }
            }catch (e:Error) {
                // screen DPI is not available
            }
            var i:int;
            for(i = 0; i < leftGroup.length; i++){
                btn = leftGroup[i];
                if(btn.getChildByName("label")){
                    btn.getChildByName("label").removeFromParent(true);
                }
                btn.x = _tuningButton.x + (btn.width + topBtnMargin) * i;
                btn.y = (_headerContainer.height - btn.height) / 2;
            }

            var rightGroup:Array = [mailButton, friendsButton, notificationsButton];
            for(i = 0; i < rightGroup.length; i++){
                btn = rightGroup[rightGroup.length - 1 - i];
                if(btn.getChildByName("label")){
                    btn.getChildByName("label").removeFromParent(true);
                }
                btn.x = _muscleCarsButton.x + _muscleCarsButton.width - btn.width - (btn.width + topBtnMargin) * i;
                btn.y = (_headerContainer.height - btn.height) / 2;
            }
            drawBottomMenu();
        }
    }

    private function drawBottomMenu():void{
        var bottomList:Array = [galleryButton, garageButton, timelineButton, challengesButton, scheduleButton];
        var menuWith:Number = _muscleCarsButton.x + _muscleCarsButton.width - _tuningButton.x;
        var btnMargin:Number = 5;
        var lblMargin:Number;
        var contentWidths:Object = {};
        var i:int;
        var contentWidth:Number = 0;
        for(i = 0; i < bottomList.length; i++){
            if(bottomList[i].getChildByName("label").downSkin){
                bottomList[i].getChildByName("label").downSkin.removeFromParent(true);
                bottomList[i].getChildByName("label").downSkin = null;
                bottomList[i].getChildByName("label").validate();
            }
            bottomList[i].alpha = 1;
            bottomList[i].height = 26;
            contentWidths[bottomList[i].name] = bottomList[i].getChildByName("label").width + bottomList[i].getChildByName("icon").width;
            contentWidth += contentWidths[bottomList[i].name];
        }

        for(i = 0; i < bottomList.length; i++){
            lblMargin = (menuWith - (bottomList.length - 1) * btnMargin - contentWidth) / (4 * bottomList.length);

            bottomList[i].width = contentWidths[bottomList[i].name] + 4 * lblMargin;
            bottomList[i].getChildByName("icon").x = lblMargin;
            bottomList[i].getChildByName("icon").y = (bottomList[i].height - bottomList[i].getChildByName("icon").height) / 2;

            bottomList[i].getChildByName("label").x = bottomList[i].getChildByName("icon").x + bottomList[i].getChildByName("icon").width + lblMargin;
            bottomList[i].getChildByName("label").y = (bottomList[i].height - bottomList[i].getChildByName("label").height) / 2;
            bottomList[i].x = _tuningButton.x;

            if(i > 0){
                bottomList[i].x = bottomList[i - 1].x + bottomList[i - 1].width + btnMargin;
            }
            bottomList[i].y = stage.stageHeight - 47;
        }
    }

    private function initHeader():void {
        var verticalLayout : VerticalLayout = new VerticalLayout();
        verticalLayout.horizontalAlign = HorizontalAlign.RIGHT;
        verticalLayout.verticalAlign = VerticalAlign.MIDDLE;
        verticalLayout.gap = -4;

        _headerContainer  = new LayoutGroup();
        _headerContainer.width = stage.stageWidth;
        _headerContainer.height = 47;
        _headerContainer.layout = verticalLayout;
        addChild(_headerContainer);
    }

    public function brandsLoaded() : Boolean {
        if(_waitingForBrandsLoad) {
            Tracer.timeStamp('HomeButton', "Opened", true);
            Starling.juggler.delayCall(function () : void {
                if(_blockerView){
                    _blockerView.hide();
                }
                _blockerView = null;
            }, 1);
            return true;
        }
        return false;
    }


    private function buttonTriggeredHandler(e:Event):void {
        if(_waitingForBrandsLoad) {
            return;
        }

        switch (e.currentTarget){
            case _tuningButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Tuning");
                    AppMode.self.setModeTuning();
                break;
            case _truckConfiguratorButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Truck");
                    AppMode.self.setModeTruckConfigurator();
                break;
            case _muscleCarsButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Muscle");
                    AppMode.self.setModeAmericanMuscle();
                break;
            /*case _wheelConfiguratorButton:
                    AppMode.self.setModeWheelConfigurator();
                break;*/
        }

        _blockerView = BlockerView.show();

        _waitingForBrandsLoad = true;
        Starling.juggler.delayCall(function () : void  {
            dispatchEventWith(MODE_SELECTED);
        }, 1)

    }

    public function get userSignedIn():Boolean{
        return _userSignedIn;
    }

    public function set userSignedIn(value:Boolean):void {
        _userSignedIn = value;
        if (!stage)	{
            return;
        }
        invalidate("userSignedIn");
    }

    override public function dispose():void {
        super.dispose();


        if(_tuningButton){
            _tuningButton.removeFromParent(true);
            _tuningButton = null;
        }
        if(_truckConfiguratorButton){
            _truckConfiguratorButton.removeFromParent(true);
            _truckConfiguratorButton = null;
        }
        if(_muscleCarsButton){
            _muscleCarsButton.removeFromParent(true);
            _muscleCarsButton = null;
        }
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }
        if(_blockerView){
            _blockerView.removeFromParent(true);
            _blockerView = null;
        }
        if(_headerContainer){
            _headerContainer.removeFromParent(true);
            _headerContainer = null;
        }
        if(galleryButton){
            galleryButton.removeFromParent(true);
            galleryButton = null;
        }
        if(garageButton){
            garageButton.removeFromParent(true);
            garageButton = null;
        }
        if(timelineButton){
            timelineButton.removeFromParent(true);
            timelineButton = null;
        }
        if(challengesButton){
            challengesButton.removeFromParent(true);
            challengesButton = null;
        }
        if(scheduleButton){
            scheduleButton.removeFromParent(true);
            scheduleButton = null;
        }
        if(mailButton){
            mailButton.removeFromParent(true);
            mailButton = null;
        }
        if(friendsButton){
            friendsButton.removeFromParent(true);
            friendsButton = null;
        }
        if(notificationsButton){
            notificationsButton.removeFromParent(true);
            notificationsButton = null;
        }
        if(settingsButton){
            settingsButton.removeFromParent(true);
            settingsButton = null;
        }
        if(loginButton){
            loginButton.removeFromParent(true);
            loginButton = null;
        }
        if(newsButton){
            newsButton.removeFromParent(true);
            newsButton = null;
        }
        _selectedTuning = null;
        if(_mainMenuView){
            DisplayObjectUtil.dispose(_mainMenuView);
            _mainMenuView = null;
        }

        DisplayObjectUtil.dispose(this);
    }

    public function updateWithLanguage(language:Language) : void {
        _language = language;

        AlertView.language = language;
        CustomAlertView.language = language;
        AlertViewHL.language = language;

        if (mainMenuView){
            mainMenuView.updateWithLanguage(language);
        }
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) mainMenuView.hide();
        dispatchEventWith(msg);
    }

    public function set mainMenuView(value : MainMenuView): void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
        _mainMenuView.visible = false;
    }

    public function get mainMenuView() : MainMenuView{
        return _mainMenuView;
    }
    public function brandsScreenButtonTriggeredHandler(event : Event=null) : void{
        mainMenuView.hide(false);
        _blockerView = BlockerView.show();
        _waitingForBrandsLoad = true;
        Starling.juggler.delayCall(function () : void  {
            dispatchEventWith(MODE_SELECTED);
        }, 1)
    }

    public function homeButtonTriggeredHandler(event : Event=null) : void{

    }

    public function enterRowsState() : void{

    }

    public function get languages() : Array{
        return _languages;
    }

    public function set languages(value : Array):void {
        _languages = value;
        invalidate("languages");
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function get selectedTuning():Tuning {
        return _selectedTuning;
    }
}
}
